;################################################################################
;#										#
;# AVR-ChipBasic2 - single chip basic computer with ATmega644			#
;# binfile API									#
;# copyright (c) 2006-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License as 		#
;# published by the Free Software Foundation; either version 3 of		#
;# the License, or (at your option) any later version.				#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################
;--------------------------------------------------------------------------------
;pointer functions
;--------------------------------------------------------------------------------
;get vram address to Y
api_getvram:	ldi	YL,LOW(libmio_vram)
		ldi	YH,HIGH(libmio_vram)
		ret
		
;get sysram address to Y		
api_getsysram:	ldi	YL,LOW(libmio_ram)
		ldi	YH,HIGH(libmio_ram)
		ret	
		
;get palette address to Y		
api_getpal:	ldi	YL,LOW(libmio_palette)
		ldi	YH,HIGH(libmio_palette)
		ret	
		
;get array address to Y
api_getarray:	ldi	YL,LOW(bas_array)
		ldi	YH,HIGH(bas_array)
		ret			
		
;get charset 0 address to Z
api_getchart0:	ldi	ZL,LOW(libmio_ctable*2)
		ldi	ZH,HIGH(libmio_ctable*2)
		ret			
		
;get charset 1 address to Z
api_getchart1:	ldi	ZL,LOW(libmio_ctable*2)
		ldi	ZH,HIGH(libmio_ctable*2)
		ret	
		
;get program base address to Z
api_getbase:	pop	ZH			;get return address
		pop	ZL
		push	ZL
		push	ZH
		mov	ZL,ZH
		subi	ZL,0x48			;sub base
		ldi	ZH,0x42			;base
api_getbase_1:	subi	ZH,0xfa			;+6
		subi	ZL,0x06
		brcc	api_getbase_1
		clr	ZL
		ret
		
;--------------------------------------------------------------------------------
;settings r16=value
;--------------------------------------------------------------------------------
api_setcolor:	sts	libmio_color,r16
		ret
		
api_getcolor:	lds	r16,libmio_color
		ret
		
api_setvmode:	sts	libmio_vidmode,r16
		ret
		
api_getvmode:	lds	r16,libmio_vidmode
		ret

api_setborder:	sts	libmio_border,r16
		ret
		
api_sync:	jmp	libmio_sync

;--------------------------------------------------------------------------------
;text functions r20=value
;--------------------------------------------------------------------------------
api_clrscr:	jmp	libmio_clr
api_outchar:	jmp	libmio_char
api_outdez:	jmp	libmio_outd
api_outhex:	jmp	libmio_outh
api_gotoxy:	jmp	libmio_gxy
api_romtext:	jmp	libmio_orom
api_thistext:	jmp	libmio_othis
api_cbox:	jmp	libmio_cbox
api_scroll:	jmp	libmio_scrl
api_newline:	jmp	libmio_newline

;--------------------------------------------------------------------------------
;keyboard functions r20=value
;--------------------------------------------------------------------------------
api_waitkey:	jmp	libmio_waitkey
api_nokey:	jmp	libmio_nokey
api_kstate:	lds	tempreg1,libmio_kbdstate
		ret
api_getkey:	lds	tempreg1,libmio_keycode
		ret
api_lastkey:	lds	tempreg1,libmio_lastkey
		ret
api_scancode:	lds	tempreg1,libmio_scancode
		ret
		
;-------------------------------------------------------------------------------
; IO functions
;-------------------------------------------------------------------------------
api_putser:	jmp	libmio_pser
api_getser:	jmp	libmio_gser
api_getserb:	jmp	libmio_gserb
api_putpar:	jmp	libmio_ppar
api_putxm:	jmp	libmio_txbl
api_getxm:	jmp	libmio_rxbl

;-------------------------------------------------------------------------------
; graphic functions
;-------------------------------------------------------------------------------
api_plot:	jmp	libmio_plot
api_draw:	jmp	libmio_draw
api_box:	jmp	libmio_box
api_fbox:	jmp	libmio_fbox
api_circle:	jmp	libmio_circle
api_fcircle:	jmp	libmio_fcircle
api_bcopy:	jmp	libmio_bcopy
api_clearvec:	jmp	libmio_cvec
api_clearvect:	jmp	libmio_ctvec

;-------------------------------------------------------------------------------
; GUI
;-------------------------------------------------------------------------------
api_menu:	jmp	libmio_dmenu
api_drawmenu:	jmp	libmio_domenu
api_alert:	jmp	libmio_abox1
api_ask:	jmp	libmio_ynbox1
api_alertthis:	jmp	libmio_abox
api_askthis:	jmp	libmio_ynbox

;-------------------------------------------------------------------------------
; sound
;-------------------------------------------------------------------------------
api_note:	jmp	libmio_note

;-------------------------------------------------------------------------------
; I2C
;-------------------------------------------------------------------------------
api_i2c_read:	jmp	libi2c_read
api_i2c_write:	jmp	libi2c_write
api_i2c_start:	jmp	libi2c_start
api_i2c_stop:	jmp	libi2c_stop
api_i2c_rbyte:	jmp	libi2c_rbyte
api_i2c_rbyten:	jmp	libi2c_rbyten
api_i2c_wbyte:	jmp	libi2c_wbyte
api_i2c_rlm75:	jmp	libi2c_rlm75

;-------------------------------------------------------------------------------
; EEPROM
;-------------------------------------------------------------------------------
api_eep_read:	jmp	libeep_read
api_eep_write:	jmp	libeep_write
api_eep_mread:	jmp	libeep_get
api_eep_mwrite:	jmp	libeep_put

;-------------------------------------------------------------------------------
; Dataflash
;-------------------------------------------------------------------------------
api_fs_format:	jmp	libdfl_format
api_fs_create:	jmp	libdfl_create
api_fs_delete:	jmp	libdfl_del

api_fs_read:	jmp	libdfl_read
api_fs_write:	jmp	libdfl_write
api_fs_cfree:	jmp	libdfl_cfree
api_fs_size:	jmp	libdfl_size
api_fs_maxpage:	jmp	libdfl_maxpage
api_fs_checkf:	jmp	libdfl_cif

api_fs_gtype:	call	libdfl_ghead
		jmp	libdfl_gtype
		
api_fs_rread:	call	libdfl_bread
		jmp	libdfl_rdbuf
		
api_fs_rwrite:	call	libdfl_wrbuf
		jmp	libdfl_bwrite
		
api_fs_gword:	jmp	libdfl_gdword


